@@ -51,7 +51,8 @@ using std::pair;
 #include "vector_hash-t.hpp"
 #include "check_list.hpp"
 #include "lsort.hpp"
-#include "iostream.hpp"
+
+#include "gettext.h"
 
 typedef unsigned int   u32int;
 static const u32int u32int_max = (u32int)-1;
