@@ -32,6 +32,9 @@
 // of how the words sounds. It is not the phoneme of the word by any
 // means.  For more information on the metaphone algorithm please see
 // the file metaphone.cc which included a detailed description of it.
+//
+// NOTE: It is assumed that that strlen(soundslike) <= strlen(word)
+//       for any possible word
 
 // POSSIBLE OPTIMIZATION:
 //   store the number of letters that are the same as the previous 
@@ -60,6 +63,8 @@
 #include "vararray.hpp"
 #include "string_list.hpp"
 
+#include "gettext.h"
+
 //#include "iostream.hpp"
 //#define DEBUG_SUGGEST
 
@@ -97,6 +102,7 @@ namespace {
   struct ScoreWordSound {
     char * word;
     char * word_clean;
+    //unsigned word_size;
     const char * soundslike;
     int           score;
     int           word_score;
@@ -162,7 +168,7 @@ namespace {
     int threshold;
     int try_harder;
 
-    EditDist (* edit_dist_fun)(const char *, const char *, 
+    EditDist (* edit_dist_fun)(const char *, const char *,
                                const EditDistanceWeights &);
 
     unsigned int max_word_length;
@@ -285,6 +291,9 @@ namespace {
 
   void Working::get_suggestions(NearMissesFinal & sug) {
 
+    if (original.word.size() * parms->edit_distance_weights.max >= 0x8000)
+      return; // to prevent overflow in the editdist functions
+
     near_misses_final = & sug;
 
     try_split();
@@ -479,6 +488,9 @@ namespace {
                              int w_score, int sl_score, 
                              bool count, WordEntry * rl)
   {
+    if (word_size * parms->edit_distance_weights.max >= 0x8000) 
+      return; // to prevent overflow in the editdist functions
+
     if (w_score < 0) w_score = LARGE_NUM;
     if (sl_score < 0) sl_score = LARGE_NUM;
     if (!sp->have_soundslike) {
@@ -490,6 +502,7 @@ namespace {
     ScoreWordSound & d = near_misses.front();
     d.word = word;
     d.soundslike = sl;
+    //d.word_size = word_size;
     
     if (parms->use_typo_analysis) {
       unsigned int l = word_size;
@@ -993,7 +1006,7 @@ namespace {
     int try_for = (parms->word_weight*parms->edit_distance_weights.max)/100;
     while (true) {
       try_for += (parms->word_weight*parms->edit_distance_weights.max)/100;
-	
+
       // put all pairs whose score <= initial_limit*max_weight
       // into the scored list
 
@@ -1006,17 +1019,16 @@ namespace {
         //            i->word_score, i->soundslike_score);
 
         if (i->word_score >= LARGE_NUM) {
-
-            int sl_score = i->soundslike_score < LARGE_NUM ? i->soundslike_score : 0;
-            int level = needed_level(try_for, sl_score);
-            
-            if (level >= int(sl_score/parms->edit_distance_weights.min))
-              i->word_score = edit_distance(original.clean,
-                                            i->word_clean,
-                                            level, level,
-                                            parms->edit_distance_weights);
+          int sl_score = i->soundslike_score < LARGE_NUM ? i->soundslike_score : 0;
+          int level = needed_level(try_for, sl_score);
+          
+          if (level >= int(sl_score/parms->edit_distance_weights.min)) 
+            i->word_score = edit_distance(original.clean,
+                                          i->word_clean,
+                                          level, level,
+                                          parms->edit_distance_weights);
         }
-
+        
         if (i->word_score >= LARGE_NUM) goto cont1;
 
         if (i->soundslike_score >= LARGE_NUM) 
@@ -1378,7 +1390,7 @@ namespace aspeller {
       limit = 1000;
       ngram_threshold = 1;
     } else {
-      return make_err(bad_value, "sug-mode", mode, "one of ultra, fast, normal, slow, or bad-spellers");
+      return make_err(bad_value, "sug-mode", mode, _("one of ultra, fast, normal, slow, or bad-spellers"));
     }
     if (!sp->have_soundslike) {
       if (try_scan_2) {
