@@ -238,8 +238,8 @@ static inline void set_sl(WordEntry & res, Str w)
 
 static void soundslike_next(WordEntry * w)
 {
-  const Str * i   = (const Str *)(w->intr[0]);
-  const Str * end = (const Str *)(w->intr[1]);
+  const Str * & i   = (const Str * &)(w->intr[0]);
+  const Str *   end = (const Str *  )(w->intr[1]);
   set_word(*w, *i);
   ++i;
   if (i == end) w->adv_ = 0;
@@ -625,6 +625,8 @@ bool WritableReplDict::soundslike_lookup(const WordEntry & word, WordEntry & o)
   } else {
     o.what = WordEntry::Misspelled;
     o.word = word.word;
+    o.word_size = word.word_size;
+    o.aff = "";
   }
   return true;
 }
@@ -662,8 +664,8 @@ WritableReplDict::Enum * WritableReplDict::detailed_elements() const {
 
 static void repl_next(WordEntry * w)
 {
-  const Str * i   = (const Str *)(w->intr[0]);
-  const Str * end = (const Str *)(w->intr[1]);
+  const Str * & i   = (const Str * &)(w->intr[0]);
+  const Str *   end = (const Str *  )(w->intr[1]);
   set_word(*w, *i);
   ++i;
   if (i == end) w->adv_ = 0;
